# -------------------------------------------------------------------------- #
# Script : vdm.tcl
# Auteur : ealexp. D'aprs un script de Galdinx et MenzAgitat.
# Version : 2.1
# Date de dernire modification : 17 octobre 2010
# -------------------------------------------------------------------------- #

# -------------------------------------------------------------------------- #
# Prrequis : 
# Ce script ncssite : 
#    - le package http
#    - Tcl 8.5
#
# La recherche sur le site ncssite un Eggdrop 1.6.20 au moins.
# En effet, pour contourner un bug du site, le script utilise les commandes
# after et vwait (pour attendre 1 seconde avant de rcuprer la page des
# rsultats). Ces commandes ne sont pas compatibles avec les anciennes
# versions d'eggdrop et bloqueront le bot.
#
# La recherche sera automatiquement dsactive si la version de votre Eggdrop
# est infrieure  1.6.20.
# Pour dfinitivement dsactiver la recherche, mettez la variable
# "recherche_activee"  0.
# -------------------------------------------------------------------------- #

# -------------------------------------------------------------------------- #
# Activation :
# Pour activer le script, il faut taper .chanset #salon +vdm en partyline.
#
# Le script fonctionnera trs bien avec les rglages par dfaut.
# Il est cependant possible de modifier certains rglages. Pour cela,
# reportez vous  la partie CONFIGURATION plus bas.
# -------------------------------------------------------------------------- #

# -------------------------------------------------------------------------- #
# Utilisation : 
# Ce script permet de rcuprer des citations du site VDM.
# La commande !vdm peut tre utilise de plusieurs manires : 

# 1) !vdm
#  Afficher une VDM alatoire.

# 2) !vdm <numro>
#  Afficher la VDM n<numro>.

# 3) !vdm last
#  Afficher la dernire VDM parue.

# 4) !vdm <chane>
#  Rechercher la chane <chane> sur le site.
# -------------------------------------------------------------------------- #

# -------------------------------------------------------------------------- #
# Changements :
#  - 1.0 : 
#   - 1re version
#  - 1.1 : 
#   - Correction suite  un changement de payload du site + simplification des regsub
#  - 2.0 :
#   - Correction suite  un changement du payload du site.
#   - Commandes !vdm last[- nombre] et !vdm <numro>. 
#   - Paramtres supplmentaires (urls du site, et couleurs de la VDM).
#  - 2.1 :
#   - Recherche sur le site.
#   - Mise en cache des VDM alatoires.
# -------------------------------------------------------------------------- #

# -------------------------------------------------------------------------- #
# LICENCE:
#     This program is free software: you can redistribute it and/or modify
#     it under the terms of the GNU General Public License as published by
#     the Free Software Foundation, either version 3 of the License, or
#     (at your option) any later version.
# 
#     This program is distributed in the hope that it will be useful,
#     but WITHOUT ANY WARRANTY; without even the implied warranty of
#     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#     GNU General Public License for more details.
# 
#     You should have received a copy of the GNU General Public License
#     along with this program.  If not, see <http://www.gnu.org/licenses/>.
# -------------------------------------------------------------------------- #

if {[catch {package require http}]} {
	putlog "\002VDM\002 : Vous avez besoin du package http pour pouvoir utiliser ce script."
	return
}

if {[catch {package require Tcl 8.5}]} {
	putlog "\002VDM\002 : Vous avez besoin de Tcl en version 8.5 au moins pour pouvoir utiliser ce script."
	return
}

if {[info commands vdm::uninstall] eq "::vdm::uninstall"} {
	vdm::uninstall
}

namespace eval ::vdm {
	#######################
	#                     #
	#    CONFIGURATION    #
	#                     #
	#######################
	
	########################################
	# COMMANDES PUBLIQUES ET AUTORISATIONS #
	########################################

	# Nom de la commande publique
	# Valeur par dfaut : "!vdm"
	variable vdm_command_name "!vdm"

	# Autorisations pour la commande
	# Valeur par dfaut "-|-"
	variable vdm_command_flags "-|-"
	
	###################
	# PARAMTRES HTTP #
	###################

	# UserAgent
	# Valeur par dfaut : "Mozilla/5.0 (Windows; U; Windows NT 5.1; en-US; rv:1.8.1.1) Gecko/20061204 Firefox/2.0.0.1"
	variable useragent "Mozilla/5.0 (Windows; U; Windows NT 5.1; en-US; rv:1.8.1.1) Gecko/20061204 Firefox/2.0.0.1"
	
	# Url pour rcuprer une VDM alatoire
	# Valeur par dfaut : "http://www.viedemerde.fr/aleatoire"
	variable url_aleatoire "http://www.viedemerde.fr/aleatoire"

	# Url pour rcuprer une seule VDM
	# Valeur par dfaut : "http://www.viedemerde.fr/inclassable/"
	variable url_une "http://www.viedemerde.fr/inclassable/"

	# Url pour rcuprer la dernire VDM
	# Valeur par dfaut : "http://www.viedemerde.fr/"
	variable url_dernieres "http://www.viedemerde.fr/"
	
	# Url de la recherche
	# Valeur par dfaut : "http://www.viedemerde.fr/apps/search.php?type=article"
	variable url_recherche "http://www.viedemerde.fr/apps/search.php?type=article"
	
	#################
	# MISE EN FORME #
	#################
	
	# Note : les couleurs sont  mettre sous la forme "<avant-plan>[,arrire-plan]".
	#        Si vous ne voulez pas de couleurs, mettez une chane vide ("")

	# Couleur du texte des VDM
	# Valeur par dfaut : ""
	variable couleur_texte_vdm "00,01"
	
	# Couleur du texte des erreurs
	# Valeur par dfaut : "14"
	variable couleur_texte_erreurs "14"
	
	# Affichage de l'ID, de la date ou de l'auteur de la VDM (dans l'ordre).
	# Valeur par dfaut : "1 1 1"
	# Exemples : "1 1 0" pour afficher l'id, la date mais pas l'auteur de la VDM.
	variable affichage "1 1 1"

	# Couleur de l'ID de la VDM
	# Valeur par dfaut : "01,15"
	variable couleur_id_vdm "01,15"
	
	# Couleur de la date de la VDM
	# Valeur par dfaut : ""
	variable couleur_date_vdm ""
	
	# Couleur de l'auteur de la VDM
	# Valeur par dfaut : ""
	variable couleur_auteur_vdm ""

	# Nombre de caractres maximum par ligne
	# Valeur par dfaut : 420
	variable max_line_length 420

	##############################
	# PARAMETRES DE L'ANTI-FLOOD #
	##############################
	
	# Anti-flood (0 = dsactiv, 1 = activ)
	variable antiflood 1

	# Combien de commandes sont autorises en combien de temps ?
	# exemple : "4:45" = 4 commandes maximum en 45 secondes;
	# les suivantes seront ignores.
	variable cmdflood_vdm "4:45"
	
	# Intervalle de temps minimum entre l'affichage de 2 messages
	# avertissant que l'anti-flood a t dclench (ne rglez pas
	# cette valeur trop bas afin de ne pas tre flood par les messages
	# d'avertissement de l'anti-flood...)
	variable antiflood_msg_interval 20

	#############
	# RECHERCHE #
	#############
	
	# La recherche sur le site VDM est bugge. Parfois, le script ne renvoie
	# tout simplement pas les bons rsultats. Pour y palier, le script
	# contient certains "hacks". Tout est configurable ci-dessous.

	# Activer la recherche
	# Valeur par dfaut : 1
	variable recherche_activee 1
	
	# Activer le rechargement de la page de rsultats si elle est corrompue
	# Valeur par dfaut : 1
	variable recherche_recharger_si_corrompu 1

	# Nombre maximum de rechargement d'une page de rsultats
	# Valeur par dfaut : 2
	variable recherche_max_iteration 2

	# Temps  attendre avant de recharger une page de rsultats (en ms)
	# Nombre par dfaut : 1000
	variable recherche_delai 1000
	
	# Temps  attendre avant de charger la page des rsultats. Mettez  0 pour un chargement immdiat.
	# Valeur par dfaut : 1000
	variable recherche_delai_avant_chargement 1000
	
	#######################################################################
	#                                                                     #
	#    NE RIEN MODIFIER APRS CE CADRE SI VOUS NE CONNAISSEZ PAS TCL    #
	#                                                                     #
	#######################################################################	
	
	# Le nom et la version actuelle du script.
	variable script_name "VDM"
	variable script_version "2.1"
	
	# Le mode debug.
	variable debug 1
	
	# Le nombre de citations par page.
	variable nb_vdm_page 13

	# Flag udef
	setudef flag vdm
	
	# Cache
	variable cache {}
	
	# Les variables de l'antiflood.
	variable current_flood {}
	variable antiflood_msg {}
	
	if {[lindex $::version 1] < 1062000} {
		# Si on a une version d'Eggdrop non-compatible, on dsactive la recherche.
		variable recherche_activee 0
		putlog "\[$script_name\] La recherche a t dsactive, car votre version d'Eggdrop n'est pas compatible avec. Consultez l'en-tte du script pour plus d'informations."
	}

	proc uninstall {args} {
		putlog "Dsallocation des ressources de \002$::vdm::script_name\002..."
		foreach binding [lsearch -inline -all -regexp [binds *[set ns [string range [namespace current] 2 end]]*] " \{?(::)?$ns"] {
			unbind [lindex $binding 0] [lindex $binding 1] [lindex $binding 2] [lindex $binding 4]
		}
		foreach running_timer [timers] {
			if { [string match "*[namespace current]::*" [lindex $running_timer 1]] } { killtimer [lindex $running_timer 2] }
		}
		foreach running_utimer [utimers] {
			if { [string match "*[namespace current]::*" [lindex $running_utimer 1]] } { killutimer [lindex $running_utimer 2] }
		}
		namespace delete [namespace current]
	}
}

# ::vdm::vdm_command
#
# Procdure principale de la commande !vdm.

proc ::vdm::vdm_command {nick host handle chan arg} {
	if {![channel get $chan vdm]} {
		return
	}
	if {($::vdm::antiflood == 1) && ([::vdm::antiflood $chan] == "flood")} {
		return
	}
	
	set arg [split $arg]

	if {[lindex $arg 0] eq ""} {
		# VDM alatoire.
		set vdm [get_newest_vdm_from_cache]
	} elseif {[string is integer [lindex $arg 0]]} {
		# Une VDM en particulier.
		set page_content [lindex [::vdm::geturl $::vdm::url_une$arg] 0]

		set vdm [lindex [::vdm::parse_page $page_content] 0]
	} elseif {$arg eq "last"} {
		# La dernire VDM parue.
		set page_content [lindex [::vdm::geturl $::vdm::url_dernieres] 0]

		set vdm [lindex [::vdm::parse_page $page_content] 0]
	} elseif {[regexp {last-(.*)} $arg dummy nb]} {
		if {$nb > $::vdm::nb_vdm_page} {
			puthelp "PRIVMSG $chan :[c $::vdm::couleur_texte_erreurs $chan]L'index doit tre infrieur ou gal  $::vdm::nb_vdm_page.[c endcolor $chan]"
			return
		}
		set page_content [lindex [::vdm::geturl $::vdm::url_dernieres] 0]

		set vdm [lindex [::vdm::parse_page $page_content] $nb]
	} else {
		# Si la recherche n'est pas active, on renvoie un message d'erreur.
		if {!$::vdm::recherche_activee} {
			puthelp "PRIVMSG $chan :[c $::vdm::couleur_texte_erreurs $chan]Utilisation : !vdm \[numro/last\[- 1  $::vdm::nb_vdm_page\]\][c endcolor $chan]"
			return	
		}
		
		# Sinon, on commence la recherche.
		lassign [::vdm::geturl $::vdm::url_recherche -query [::http::formatQuery texte $arg] -timeout 6000] data url

		for {set i 0} {$i < $::vdm::recherche_max_iteration} {incr i} {
			debug "Entre dans une nouvelle itration"
			
			if {[string match "*les recherches $arg*" $data]} {
				set vdm [::vdm::parse_page $data]
				set vdm [lindex $vdm [rand [llength $vdm]]]
				break
			}
			
			if {!$::vdm::recherche_recharger_si_corrompu} {
				break
			}

			# Si on est l c'est que la page de recherche qu'on nous a envoy est corrompue.
			# Donc on attend un peu de temps et on recharge la page.
			after $::vdm::recherche_delai {
				set trigger 1
			}
			vwait trigger

			set data [lindex [::vdm::geturl $url] 0]
		}
		if {![info exists vdm]} {
			debug "Recherche choue"
			# Si la variable vdm n'existe pas, c'est que malgr nos tentatives de rechargement, la recherch a chou.
			# On assigne donc  vdm la valeur "search_error".
			set vdm "search_error"
		}

	}
	
	switch $vdm {
		"http_error" {
			puthelp "PRIVMSG $chan :[c $::vdm::couleur_texte_erreurs $chan]La connexion  VDM n'a pas pu tre tablie. Peut-tre le site rencontre-t-il des difficults techniques. Veuillez ressayer plus tard.[c endcolor $chan]"
			return
		}

		"vdm_error" {
			puthelp "PRIVMSG $chan :[c $::vdm::couleur_texte_erreurs $chan]La VDM n'a pas pu tre rcupre.[c endcolor $chan]"
			return
		}

		"search_error" {
			puthelp "PRIVMSG $chan :[c $::vdm::couleur_texte_erreurs $chan]La recherche n'a pas pu tre effectue. Veuillez resayer plus tard.[c endcolor $chan]"
			return
		}
	}
	
	lassign $::vdm::affichage afficher_id afficher_date afficher_auteur
	lassign $vdm id message date auteur

	# Affichage de l'ID 
	if {$afficher_id} {
		append privmsg "[c $::vdm::couleur_id_vdm $chan]\[$id\][c endcolor $chan] "
	}

	# La VDM en elle-mme
	append privmsg "[c $::vdm::couleur_texte_vdm $chan]$message[c endcolor $chan]"

	# Affichage de la date
	if {$afficher_date} {
		append privmsg " - [c $::vdm::couleur_date_vdm $chan]$date[c endcolor $chan]"
	}
	
	# Affichage de l'auteur
	if {$afficher_auteur} {
		append privmsg " - par [c $::vdm::couleur_auteur_vdm $chan]$auteur[c endcolor $chan]"
	}

	foreach ligne [::vdm::wrap $privmsg $::vdm::max_line_length] {
		puthelp "PRIVMSG $chan :$ligne"
	}
	return
}

# ::vdm::parse_page
#
# Parse une page et rcupre toutes les VDM qui y figurent.
# Retourne par dfaut la xime VDM trouve. Si le paramtre index est "random", retourne une VDM au hasard.

proc ::vdm::parse_page {data {index 0}} {
 	foreach {dummy id vdm date auteur} [regexp -all -inline {<div class="post" id="(.+?)">(.+?)</p><div class="date">(?:.*?)<p style="margin-top:(?:.*?)">(.*?) - <a class="(?:.*?)" href="(?:.*?)">(?:.*?)</a> - par (.*?) } $data] {
		lappend vdm_list [list $id [::vdm::string_filter $vdm] $date $auteur]
	}
	
	if {![info exists vdm_list]} {
		return "vdm_error"
	}
	
	return $vdm_list
}

# ::vdm::valid_cache
#
# Vrifie que le cache n'est pas vide ou qu'on n'a pas atteint la fin du cache.

proc ::vdm::usable_cache {} {
	lassign $::vdm::cache last_vdm_read vdm_list
	return [expr {$last_vdm_read ne "" && $last_vdm_read != [llength $vdm_list]}]
}

# ::vdm::feed_cache
#
# Efface tout le contenu du cache et y met de nouvelles VDM.

proc ::vdm::feed_cache {} {
	set page_content [lindex [::vdm::geturl $::vdm::url_aleatoire] 0]
	set vdm_list [::vdm::parse_page $page_content]

	set ::vdm::cache [list -1 $vdm_list]

	return 1
}

# ::vdm::get_new_vdm_from_cache
#
# Vrifie si le cache est encore valide. Si non, remplit le cache. Puis retourne la premire VDM non-lue du cache.

proc ::vdm::get_newest_vdm_from_cache {} {
	if {![::vdm::usable_cache]} {
		::vdm::feed_cache	
	}
	
	lassign $::vdm::cache last_vdm_read vdm_list
	lset ::vdm::cache 0 [expr {$last_vdm_read + 1}]
	
	return [lindex $vdm_list $last_vdm_read+1]
}

# ::vdm::string_filter
#
# Nettoie la VDM des entits et balises HTML.

proc ::vdm::string_filter {str} {
	set str [string map -nocase {
		"&agrave;"			""		"&agrave;"			""		"&aacute;"			""		"&acirc;"			""
		"&atilde;"			""		"&auml;"			""		"&aring;"			""		"&aelig;"			""
		"&ccedil;"			""		"&egrave;"			""		"&eacute;"			""		"&ecirc;"			""
		"&euml;"			""		"&igrave;"			""		"&iacute;"			""		"&icirc;"			""
		"&iuml;"			""		"&eth;"				""		"&ntilde;"			""		"&ograve;"			""
		"&oacute;"			""		"&ocirc;"			""		"&otilde;"			""		"&ouml;"			""
		"&divide;"			""		"&oslash;"			""		"&ugrave;"			""		"&uacute;"			""
		"&ucirc;"			""		"&uuml;"			""		"&yacute;"			""		"&thorn;"			""
		"&yuml;"			""		"&quot;"			"\""	"&amp;"				"&"		"&euro;"			""
		"&oelig;"			""		"&Yuml;"			""		"&nbsp;"			" "		"&iexcl;"			""
		"&cent;"			""		"&pound;"			""		"&curren;"			""		"&yen;"				""
		"&brvbar;"			""		"&brkbar;"			""		"&sect;"			""		"&uml;"				""
		"&die;"				""		"&copy;"			""		"&ordf;"			""		"&laquo;"			""
		"&not;"				""		"&shy;"				"-"	"&reg;"				""		"&macr;"			""
		"&hibar;"			""		"&deg;"				""		"&plusmn;"			""		"&sup2;"			""
		"&sup3;"			""		"&acute;"			""		"&micro;"			""		"&para;"			""
		"&middot;"			""		"&cedil;"			""		"&sup1;"			""		"&ordm;"			""
		"&raquo;"			""		"&frac14;"			""		"&frac12;"			""		"&frac34;"			""
		"&iquest;"			""		"&Agrave;"			""		"&Aacute;"			""		"&Acirc;"			""
		"&Atilde;"			""		"&Auml;"			""		"&Aring;"			""		"&AElig;"			""
		"&Ccedil;"			""		"&Egrave;"			""		"&Eacute;"			""		"&Ecirc;"			""
		"&Euml;"			""		"&Igrave;"			""		"&Iacute;"			""		"&Icirc;"			""
		"&Iuml;"			""		"&ETH;"				""		"&Dstrok;"			""		"&Ntilde;"			""
		"&Ograve;"			""		"&Oacute;"			""		"&Ocirc;"			""		"&Otilde;"			""
		"&Ouml;"			""		"&times;"			""		"&Oslash;"			""		"&Ugrave;"			""
		"&Uacute;"			""		"&Ucirc;"			""		"&Uuml;"			""		"&Yacute;"			""
		"&THORN;"			""		"&szlig;"			""		"\r"				""		"\t"				""
		"&#039;"			"\'"	"&#39;"				"\'"	"&gt;"				">"		"&lt;"				"<"
		"&#34;"				"\'"	"&#38;"				"&"		"#91;"				"\("	"&#92;"				"\/"
		"&#93;"				")"		"&#123;"			"("		"&#125;"			")"		"&#163;"			""
		"&#168;"			""		"&#169;"			""		"&#171;"			""		"&#173;"			""
		"&#174;"			""		"&#180;"			""		"&#183;"			""		"&#185;"			""
		"&#187;"			""		"&#188;"			""		"&#189;"			""		"&#190;"			""
		"&#192;"			""		"&#193;"			""		"&#194;"			""		"&#195;"			""
		"&#196;"			""		"&#197;"			""		"&#198;"			""		"&#199;"			""
		"&#200;"			""		"&#201;"			""		"&#202;"			""		"&#203;"			""
		"&#204;"			""		"&#205;"			""		"&#206;"			""		"&#207;"			""
		"&#208;"			""		"&#209;"			""		"&#210;"			""		"&#211;"			""
		"&#212;"			""		"&#213;"			""		"&#214;"			""		"&#215;"			""
		"&#216;"			""		"&#217;"			""		"&#218;"			""		"&#219;"			""
		"&#220;"			""		"&#221;"			""		"&#222;"			""		"&#223;"			""
		"&#224;"			""		"&#225;"			""		"&#226;"			""		"&#227;"			""
		"&#228;"			""		"&#229;"			""		"&#230;"			""		"&#231;"			""
		"&#232;"			""		"&#233;"			""		"&#234;"			""		"&#235;"			""
		"&#236;"			""		"&#237;"			""		"&#238;"			""		"&#239;"			""
		"&#240;"			""		"&#241;"			""		"&#242;"			""		"&#243;"			""
		"&#244;"			""		"&#245;"			""		"&#246;"			""		"&#247;"			""
		"&#248;"			""		"&#249;"			""		"&#250;"			""		"&#251;"			""
		"&#252;"			""		"&#253;"			""		"&#254;"			""		
		
	} $str]
	regsub -all {<br />} $str " " str
	regsub -all {<[^>]+>} $str "" str
	regsub -all {\n} $str " " str

	return $str
}

# ::vdm::geturl
#
# Appelle ::http::geturl et suit les redirections.
# Retourne le contenu de la page et son URL

proc ::vdm::geturl {url args} {
	::http::config -useragent $::vdm::useragent
	
	# On rcupre l'hte de l'URL, au cas o il ne serait pas donn par le
	# site dans le chemin de la redirection.
	set original_url_host [lindex [regexp -inline {http://(.*?)/} $url] 0]
	
	# On suivra au maximum 5 redirections.
	for {set i 0} {$i < 5} {incr i} {
		# Si c'est la recherche qu'on doit rcuprer et que l'option
		# d'attente avant la rcupration de la page est active,
		# on attend.
		if {[string match "*recherche/resultat*" $url]} {
			after $::vdm::recherche_delai_avant_chargement {
				set trigger 1
			}
			vwait trigger
		}

		set token [::http::geturl $url {*}$args]
		if {![string match {30[1237]} [::http::ncode $token]]} {
			break
		}

		set meta [set ${token}(meta)]
		::http::cleanup $token
		
		if {![dict exists $meta Location]} {
			break
		}

		set url [dict get $meta Location]
		
		# Si le site n'a pas indiqu d'hte dans le chemin, on utilise
		# l'hte rcupr au dbut.
		if {[regexp -inline {http://(.*?)/} $url] eq ""} {
			set url $original_url_host$url
		}
	}

	set data [::http::data $token]
	::http::cleanup $token

	return [list $data $url]
}

# ::vdm::antiflood
#
# Vrifie si l'antiflood n'est pas dclench sur le chan.

proc ::vdm::antiflood {chan} {
	if {![dict exists $::vdm::current_flood $chan]} {
		dict set ::vdm::current_flood $chan 0
	}
	if {![dict exists $::vdm::antiflood_msg $chan]} {
		dict set ::vdm::antiflood_msg $chan 0
	}

	set max_requests [lindex [split $::vdm::cmdflood_vdm ":"] 0]
	set time [lindex [split $::vdm::cmdflood_vdm ":"] 1]
	
	if {[dict get $::vdm::current_flood $chan] >= $max_requests} {
		if {[dict get $::vdm::antiflood_msg $chan] == 0} {
			dict set ::vdm::antiflood_msg $chan 1
			putquick "PRIVMSG $chan :[c 7 $chan]:::[c $::vdm::couleur_texte_erreurs $chan] Contrle de flood activ pour la commande [c bold $chan]$::vdm::vdm_command_name[c bold $chan] : pas plus de $max_requests requte(s) toutes les $time secondes.[c endcolor $chan]"
			utimer $::vdm::antiflood_msg_interval [list ::vdm::antiflood_msg_reset $chan]
		}
		return "flood"
	} else {
		dict incr ::vdm::current_flood $chan
		utimer $time [list ::vdm::antiflood_close_instance $chan]
		return "no flood"
	}
}

# ::vdm::antiflood_close_instance

proc ::vdm::antiflood_close_instance {chan} {
	if {[dict get $::vdm::current_flood $chan] > 0} {
		dict incr ::vdm::current_flood $chan -1
	}
}

# ::vdm::antiflood_msg_reset

proc ::vdm::antiflood_msg_reset {chan} {
	dict set ::vdm::antiflood_msg $chan 0
}


# ::vdm::c
# 
# Vrifie que les codes couleurs sont autoriss sur le salon pass en
# argument et renvoie le code couleur demand.

proc ::vdm::c {code chan} {
	if {[string match *c* [lindex [split [getchanmode $chan]] 0]] || $code eq ""} {
		return ""
	}
	
	switch -- $code {
		"bold" {
			return \002
		}

		"underline" {
			return \037
		}

		"endcolor" {
			return \003
		}

		default {
			return \003$code
		}
	}
}


# ::vdm::wrap
#
# Coupe une chane tous les $width caractres.
# Si elle peut, elle coupe entre deux mots, sinon, elle peut couper un mot en
# deux.

proc ::vdm::wrap {text {width 1}} {
	incr width -1
	set text [string trim $text]
	set text_length [string length $text]
	
	if {$width < 0} {
		error "width doit tre positif"
	}
	
	if {$text_length <= $width} {
		return [list $text]
	} else {
		set cursor 0
		while {$cursor < $text_length} {
			if {([set cut_index [::tcl::string::last " " $text [expr {$cursor+$width}]]] == -1) || ($cut_index <= $cursor) || ($text_length - $cursor < $width)} {
			    set cut_index [expr {$cursor + $width}]
			}

			lappend output [::tcl::string::range $text $cursor $cut_index]
			set cursor [expr {$cut_index+1}]
		}   
		return $output
	}   
}

# ::vdm::debug
#
# Affiche un message en partyline si le mode debug est activ.

proc ::vdm::debug {args} {
	if {!$::vdm::debug} {
		return
	}
	putlog "\[$::vdm::script_name\] [join $args { }]"
}

bind evnt -|- prerehash ::vdm::uninstall
bind pub $::vdm::vdm_command_flags $::vdm::vdm_command_name ::vdm::vdm_command
